 # -*- coding: utf-8 -*-
"""
Spyder Editor

This is a temporary script file.
"""

import pyqtgraph as pg
from ADoutputs import daq_AO, daq_DO
from pyqtgraph.Qt import QtGui, QtCore
from PyQt5.QtCore import QTimer
from PyQt5.QtWidgets import (QPushButton, QWidget, QVBoxLayout, QLCDNumber, QProgressBar, QLabel, QCheckBox, QComboBox)
import telnetlib
import os, serial
from messages import show_warning

import scanFunctions 
import wav_coherent

#An example of a class
class anal_control_signal(QWidget):
    _registry = []
    
    def __init__(self, name, channel, out_minimum = -10, out_maximum = 10, out_step = 0.1, feedfoward = False):
        super().__init__()
        
        self._registry.append(self)
        
        self.name = name
        self.ch = channel

        self.is_feedwoward_enabled = feedfoward
        
        self.initial_value = 0
        
        self.max = out_maximum
        self.min = out_minimum
        self.step = out_step
        self.siPrefix = True
        self.suffix = 'V'

        self.sb = pg.SpinBox(value = self.initial_value, bounds=[self.min, self.max], step = self.step, siPrefix = self.siPrefix, suffix = self.suffix)
        try:
            self.AO = daq_AO(self.ch)
        except:
            show_warning('Analog output %i not found. Daq might not be connected' % self.ch)
        
        
        
        self.scan_button = QPushButton('Start scan')
        
        self.ScanTimer = QTimer()
        self.ScanTimer.timeout.connect(self.scanEvent)
        self.scan_step = 0
        
        self.scanpBar = QProgressBar(self)
        self.scanLabelValue = QLabel(self)
        self.PbarTimer = QTimer(self)
        self.PbarTimer.timeout.connect(self.PbarEvent)

        self.scan_sb_start =  pg.SpinBox(value = 0, bounds=[self.min, self.max], step = self.step, siPrefix = self.siPrefix, suffix = self.suffix)
        self.scan_sb_stop =    pg.SpinBox(value = 0, bounds=[self.min, self.max], step = self.step, siPrefix = self.siPrefix, suffix = self.suffix)
        self.scan_sb_period = pg.SpinBox(value = 1, bounds=[0, 10000], step = 0.1, siPrefix = True, suffix = 's')
        
        
        self.scan_button.clicked.connect(self.doScanAction)
        
        self.spin_boxes = [('sb', self.sb), ('scan_sb_start', self.scan_sb_start), ('scan_sb_stop', self.scan_sb_stop), ('scan_sb_period', self.scan_sb_period)]
        
        #set minimum height for spinboxes:
        self.sb.setMinimumHeight(22)
        self.scan_sb_period.setMinimumHeight(22)
        self.scan_sb_start.setMinimumHeight(22)
        self.scan_sb_stop.setMinimumHeight(22)
        
        self.scan_feedfoward_button = QCheckBox('Feedfoward')
        self.scan_feedfoward_sb = pg.SpinBox(value = 0, bounds=[0, 0.005], step = 0.0005)
        self.scan_feedfoward_sb.setMinimumHeight(22)
        self.scan_feedfoward_step = 0
        self.scan_feedfoward_analogOut = None
        
    def sb_analog_change(self):
        self.AO.set_out(self.sb.val)
        
    def set_name(self, name):
        self.name = name
    

        
    def set_siPrefix(self, value):
        self.siPrefix = value
    
    def set_sufix(self, value):
        self.sufix = value
    
        
    def connectSpinBoxesToDaq(self):
        print(self.name + 'Initial value: ', self.sb.val)
        self.sb_analog_change()
        self.sb.sigValueChanging.connect(self.sb_analog_change)
        
    
    def scanEvent(self):
         
        self.scan_step = next(self.scan_array_gen)
        self.AO.set_out(self.scan_step)
        
        if self.is_feedwoward_enabled and self.scan_feedfoward_button.isChecked():
            self.scan_feedfoward_step = next(self.scan_array_feedfoward_gen)
            self.AO_feedfoward.set_out(self.scan_feedfoward_step)
        

    def PbarEvent(self):
        try:
            self.scanpBar.setValue( 100 * float(self.scan_step-float(self.scan_sb_start.val))/float(self.scan_sb_stop.val-self.scan_sb_start.val)   )
            self.scanLabelValue.setText("%.2f" % self.scan_step + ' V')
        except ZeroDivisionError:
            self.doScanAction()
            show_warning('Oh dear, you might be trying to scan from 0 to 0. Plase never do that again.')
    
    def doScanAction(self):
        
                        
        if self.ScanTimer.isActive():
            self.ScanTimer.stop()
            self.scan_button.setText('Start scan')
            self.scan_button.setStyleSheet('background-color: None')
            self.PbarTimer.stop()
            self.sb.setValue(self.scan_step)
            self.sb.setEnabled(True)
            self.scan_sb_start.setEnabled(True)
            self.scan_sb_stop.setEnabled(True)
            
            
        else:
            
            dt = 5.0e-3
                        
            first_value = float(self.sb.val)            
            self.scan_array = scanFunctions.create_scan_array(self.scan_sb_start.val, self.scan_sb_stop.val, self.scan_sb_period.val, first_value, dt)
            self.scan_array_gen = scanFunctions.yield_scan_array(self.scan_array)
            
            self.scan_array_feedfoward = scanFunctions.create_scan_array(self.scan_sb_start.val, self.scan_sb_stop.val,
                                                                         self.scan_sb_period.val, first_value, dt,
                                                                         normalized=True, normalization_constant = float(self.scan_feedfoward_sb.val) )
            self.scan_array_feedfoward_gen = scanFunctions.yield_scan_array(self.scan_array_feedfoward)
            
#            import pylab as plt
#            plt.figure()
#            plt.plot(self.scan_array, label = 'scan')
#            plt.plot(self.scan_array_feedfoward, label = 'feedfoward')
#            plt.legend()
#            plt.xlabel('u.a'), plt.ylabel('Voltaje [V]')
#            plt.show()
#            
            
            if first_value > float(self.scan_sb_stop.val) or first_value < float(self.scan_sb_start.val):
                details = 'Remember remember the fifth of November.' + '\n' + 'Also remember that scan starts from the value of the corresponding spin box'
                show_warning('Scan can not start outside scan range and scan start should be lower than scan stop', details_text= details)
            elif len(self.scan_array) < 5:
                show_warning('%i steps are too few, you need at least 5 steps' % (len(self.scan_array)) )
            else:
                self.sb.setEnabled(False)
                self.scan_sb_start.setEnabled(False)
                self.scan_sb_stop.setEnabled(False)
                self.scan_button.setStyleSheet("background-color: green")
                self.PbarTimer.start(dt*1e3)
                self.ScanTimer.start(dt*1e3)
                self.scan_button.setText('Scanning')
            
                
            
    def set_scan_feedfoward_analog_out(self, feedfoward_channel):
        self.AO_feedfoward = daq_AO(feedfoward_channel)
        
        

            
class digital_control_signal(QWidget):
    
    def __init__(self, name, channel):
        super().__init__()
        
        self.name = name
        self.ch = channel
        
        self.initial_value = 0
        try:
            self.DO = daq_DO(self.ch)
        except:
            show_warning('Digital output %i not found. Daq might not be connected' % self.ch)
            
        self.cb = QtGui.QCheckBox()
        self.cb.stateChanged.connect(self.whenChangeDigital)
        
        self.blink_cb = QtGui.QCheckBox()
        self.blink_cb.stateChanged.connect(self.blink)
        
        self.blink_timer = QTimer()
        self.blink_timer.timeout.connect(self._blinkEvent)
        self.blink_variable = True
        
    def set_name(self, name):
        self.name = name
    
    def set_initial_value(self, value):
        self.initial_value = value
    
    def whenChangeDigital(self):
        self.DO.set_out(not self.cb.isChecked())
        
    def blink(self):
        
        if self.blink_cb.isChecked():
            self.blink_variable = self.cb.isChecked()
            self.blink_timer.start(1000)
            print("starting blinkin")
        else:
            self.blink_timer.stop()
            self.DO.set_out( self.cb.isChecked() )
            print("finishing blink")

    def _blinkEvent(self):
        self.blink_variable = not self.blink_variable
        self.DO.set_out(self.blink_variable)
        
        
        
        
class activation_button(QWidget):
    
    def __init__(self):
        QWidget.__init__(self)
        self.button = QPushButton('', self)
        self.button.clicked.connect(self.handleButton)
        self.button.setIcon(QtGui.QIcon(os.path.dirname(os.path.realpath(__file__)) + "\\snuffles1.png"))
        self.button.setIconSize(QtCore.QSize(48,48))
        layout = QVBoxLayout(self)
        layout.addWidget(self.button)

    def handleButton(self):
        self.button.setIcon(QtGui.QIcon(os.path.dirname(os.path.realpath(__file__))+ "\\snuffles.png"))

class burleighWM(QWidget):
        def __init__(self, layout, dock):
            super().__init__()
            
            self.dockWM = dock
            self.layoutWM = layout
            
            wavemeter_ok = False
            try:
                tn = telnetlib.Telnet(host = "wannanosaurus", port = 1234, timeout = 1)
                wavemeter_ok = True

            except:
                show_warning('Wannanosaurus not connected')

            
            
            
            if wavemeter_ok:
                tn.write(b'++eos 1\n')
    
                lcd = QLCDNumber()
            
                lcd.setDigitCount(9) 
                lcd.display("000000 00")
       
                def update_label():
                    tn.sock_avail()
                    tn.write(b'++read <cr>\n')
                    a = tn.read_very_eager().decode("utf-8")
                    #print(a)
                    try:
                        lcd.display(a[3:12])
                    except:
                        lcd.display("000000 00")
                    
                self.layoutWM.setGeometry(300, 300, 250, 150)
                
                self.layoutWM.timer = QTimer()
                self.layoutWM.timer.timeout.connect(update_label)
                self.layoutWM.timer.start(1000)  # every 10,000 milliseconds
                
                self.layoutWM.addWidget(lcd)
                self.dockWM.addWidget(self.layoutWM)
                
class coherentWM(QWidget):
        def __init__(self, layout, dock):
            super().__init__()
            
            self.dockWM = dock
            self.layoutWM = layout
            
            wavemeter_ok = False
            try:
                wavCoh= serial.Serial('COM11', timeout=1, rtscts=True)
                wavCoh.close()
                
                wav_coherent.read_measurement(wavCoh)
                
                wavemeter_ok = True
                print('coherent wavemeter connected')

            except:
                show_warning('Coherent wavemeter not connected')

            
            if wavemeter_ok:
                

                lcd = QLCDNumber()
            
                lcd.setDigitCount(11) 
                lcd.display("00000000 0")
                
                wav_coherent.set_unit(wavCoh, b'F')
                
                def update_label():
                    
                    value =  wav_coherent.read_measurement(wavCoh)
                    unit = wav_coherent.check_unit(wavCoh)
                   
                    try:
                        lcd.display(value + ' ' + unit)
                        
                    except:
                        lcd.display("error000 0")


                
                
                self.layoutWM.timer = QTimer()
                self.layoutWM.timer.timeout.connect(update_label)
                self.layoutWM.timer.start(500)  # every 10,000 milliseconds
                self.layoutWM.setGeometry(300, 300, 250, 150)

                
                    
                combo = QComboBox()
                combo.addItem("Frequency")
                combo.addItem("Wavelength Vac")
                combo.addItem("Wavelength Air")
                
                
                
                def comboActivated(text):
                    if text == "Frequency":
                        unit = b'F'
                    if text == "Wavelength Vac":
                        unit = b'V'
                    if text == "Wavelength Air":
                        unit = b'A'
                    with wavCoh as w:    
                  
                        wav_coherent.set_unit(w, unit)
                    
                combo.activated[str].connect(comboActivated)

                
      
                self.layoutWM.addWidget(lcd, 0, 0)
                self.layoutWM.addWidget(combo, 1, 0)
                self.dockWM.addWidget(self.layoutWM)    
                